import sys

from dbcfg import *
#import dbcfg
import os,json

def main():
    c=c_commandarg(globals(),"dbcfg工具、样例")
    c.main()

def 显示所有的配置文件():
    for cdir in cfgdir():
        print(f"目录{cdir}下配置文件：")
        文件清单=[]
        for f in os.listdir(cdir):
            if f.endswith(".cfg"):
                文件清单.append(f[:-4])
        文件清单.sort()
        print(" ".join(文件清单))

def info():
    '显示配置文件内容，可以用参数指定配置文件名（不用加.cfg）'
    if len(sys.argv)>2:
        配置名=sys.argv[2]
    else:
        配置名=sys.argv[1]
    dbc=dbcfg(配置名,ehm=1)    #读取指定的配置文件里的配置信息
    if dbc.rtcode!=0:
        print("参数需要指定配置文件名")
        显示所有的配置文件()
        return
#   dbc=dbcfg.use(sys.argv[1],ehm=1)    #如果使用import dbcfg引入包，就需要使用use函数
    cfg=dbc.cfg()           #返回指定名称的配置，不指定使用name为""的那一个
    print(f"cfg={cfg}")
    print(json.dumps(cfg,ensure_ascii=False,skipkeys=False,indent=2))

def test():
    '读配置文件，显示基础信息，调用相应的测试函数'
    if len(sys.argv)<3:
        print("需要附加一个参数指定配置文件名\n")
        显示所有的配置文件()
        return
    dbc=dbcfg(sys.argv[2],ehm=1)    #读取指定的配置文件里的配置信息
#   dbc=dbcfg.use(sys.argv[1],ehm=1)    #如果使用import dbcfg引入包，就需要使用use函数
    dbc.connect()
    print(f"dbc.dbname={dbc.dbname}")
    cfg=dbc.cfg()           #返回指定名称的配置，不指定使用name为""的那一个
    print(f"cfg={cfg}")
    dbc.test()

def readdb():   #'读数据代码样例'
    字段=dbc.jg1("select 字段1 from 表 where 一些条件")  #返回结果只有一行数据用jg1

    字段1,字段2=dbc.jg1("select 字段1,字段2 from 表 where 一些条件") #jg1支持多字段

    结果=dbc.jg1("select 字段1,字段2 from 表 where 一些条件")  #jg1多字段另一种用法
    字段1,字段2=结果

    字段=dbc.jg1(f"select 字段1 from 表 where 字段2='{字段2筛选值}'")  #where条件可以用f格式串加入参数
    
    for 字段1,字段2 in dbc.execute("select 字段1,字段2 from 表 where 一些条件"): #循环读取多条数据
        print(字段1,字段2)

    for 字段1, in dbc.execute("select 字段1 from 表 where 一些条件"): #注意如果返回结果只有一个字段要加个逗号，不然返回的是数组
        print(字段1)

def wiki(): #wiki机器人读配置示例代码
    import mwclient
    dbc=dbcfg("wiki")
    cfg=dbc.cfg()
    site = mwclient.Site(cfg["d"]["server"], scheme=cfg["d"]["scheme"],path=cfg["d"]["path"])
    site.login(cfg["d"]["user"],cfg["d"]["password"])

if __name__ == "__main__":
    main()



__all__=["dbcfg","c_commandarg","cfgdir"]

import os,json
import inspect,sys

def cfgdir():   #返回配置目录列表
    配置目录=[]
    home_dir = os.path.expanduser("~")
    配置目录.append(os.path.join(home_dir,".dbcfg.d"))
    dh,_=os.path.splitdrive(os.getcwd())
    配置目录.append(os.path.join(dh,"/etc","dbcfg.d"))
    配置目录.append(os.path.join(dh,"/etc","dbconn.config.d"))
    return 配置目录

class dbcfg(object):    #dbcfg主类
    def __init__(self,连接名='',ehm=0):
        self.ehm=ehm
        self.connected=False
        self.读配置文件(连接名)

    def 读配置文件(self,连接名):
        self.rtcode=0
        配置目录=cfgdir()
        self.connectname=连接名
        for 目录 in 配置目录:
            配置文件=os.path.join(目录,f"{连接名}.cfg")
            if os.path.isfile(配置文件):
                self.cfgdata=json.loads(open(配置文件).read())
                return
        self.q(-1,{"连接名":连接名})
        
    def cfg(self,实例=''):
        for 配置 in self.cfgdata:
            if not 实例 or 实例==配置.get("name",""):
                return 配置
        self.q(-2,{"实例":实例})
    def connect(self,实例=''):
        self.rtcode=0
        默认包={"oracle":"cx_Oracle","mysql":"pymysql","sqlserver":"pytds","tds":"pytds","opengauss":"py_opengauss.driver.dbapi20",
            "dm":"dmPython"}
        import importlib
        cfg=self.cfg(实例)
        self.dbname=cfg["db"]
        if "python" not in cfg or "import" not in cfg["python"]:
            包=默认包.get(cfg["db"],cfg["db"])
        else:
            包=cfg["python"]["import"]
        try:
            m=importlib.import_module(包)
        except:
            return self.q(-4,{"name":包})
        try:
            self.conn=m.connect(*cfg["t"],**cfg["d"])
            self.connected=True
        except:
            return self.q(-5)
        else:
            return self.conn

    def q(self,返回码,参数={}):
        消息表={
            -1: "未找到配置文件{连接名}.cfg",
            -2:"配置文件中未找到实例{实例}",
            -3:"配置文件中需要设置python相关内容",
            -4:"import {name}错误",
            -5:"连接到数据库错误"
        }
        self.rtcode=返回码
        self.rtinfo=消息表.get(返回码,"错误的返回码").format(**参数)
        if self.ehm==1:
            print(self.rtinfo)
        if self.ehm==2:
            raise Exception(self.rtinfo)
        return self.rtcode
    def commit(self):
        self.conn.commit()
    def execute(self,ssql,*args,**kwargs):
        c=self.conn.cursor()
        c.execute(ssql,*args,**kwargs)
        return c
    def execute2(self,ssql,*args,**kwargs):
        try:
            list = []
            c = self.conn.cursor()
            c.execute(ssql, kwargs)
            col = c.description
            for item in c.fetchall():
                dict = {}
                for i in range(len(col)):
                    dict[col[i][0]] = item[i]
                list.append(dict)
        except self.dbdriver.DatabaseError as e:
            self.connected = False
            print("oracle执行异常：%s" % (e))
            if self.raiseExp:
                raise  # 往上层抛
            return
        return list
    def jg1(self,ssql,*args,**kwargs):
        '''根据sql返回1条结果'''
        c=self.conn.cursor()
        c.execute(ssql,*args,**kwargs)
        jg=c.fetchone()
        c.close()
        if jg==None:
            return
        if len(jg)==1:
            return jg[0]
        else:
            return jg
    def xg(self,ssql,*args,**kwargs):
        '''主要用于修改，执行完后附加commit操作'''
        c=self.execute(ssql,*args,**kwargs)
        self.commit()
        return c
    def test(self):     #通过获取当前时间来测试数据库是否工作正常
        if self.dbname.lower()=="oracle":
            print(self.jg1("select sysdate from dual"))
    def __getattribute__(self,name):
        if name in ("读配置文件","connect"):
            self.code=0
            self.info=""
        if name in ("c","conn") and not self.connected:
            self.connect()
        return object.__getattribute__(self,name)
    def __del__(self):
        if self.connected:
            self.conn.close()

class c_commandarg: #根据命令行参数调用模块及显示相应help
    tcmd={}
    scriptname=""
    def __init__(self,gg,scriptname):
        c_commandarg.scriptname=scriptname
        for g in gg:
            if (inspect.isfunction(gg[g]) or inspect.isclass(gg[g])) and gg[g].__doc__!=None:
                c_commandarg.tcmd[g]=gg[g]
    def main(self):
        if len(sys.argv)>1:
            cmd=sys.argv[1]
        else:
            self.printhelp()
            return
        if cmd not in c_commandarg.tcmd:
            cmd="info"
        cf=c_commandarg.tcmd[cmd]
        if inspect.isfunction(cf):
            rtn=cf()
        else:
            c=cf()
            rtn=0
            if hasattr(c,"main"):
                rtn=c.main()
        return rtn
    
    @staticmethod
    def printhelp():
        '''显示程序说明及命令行、参数信息'''
        print(c_commandarg.scriptname)
        print("-"*len(c_commandarg.scriptname)*2)
        maxcmdsize=0
        scmd=[]
        for name in c_commandarg.tcmd:
            scmd.append(name)
            if len(name)>maxcmdsize:
                maxcmdsize=len(name)
        scmd.sort()
        for name in scmd:
            print("%-*s   %s" %(maxcmdsize,name,c_commandarg.tcmd[name].__doc__))



#!/usr/bin/env python3

import setuptools,distutils,shutil,re,os

with open("README.md", "r",encoding='utf-8') as fh:
    long_description = fh.read()

setuptools.setup(
    name="dbcfg",
    version="0.1.3",
    author="Chen chuan",
    author_email="kcchen@139.com",
    description="数据库连接信息管理",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=setuptools.find_namespace_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.7',
    zip_safe= False,
    include_package_data = True,
    entry_points={
        'console_scripts':  [
            'dbcfg=script.dbcfgtool:main',
        ],
    },
)
